(*************************************************************
 *                Auto-assemblage de graphes                 *
 *************************************************************
   Auteur  : Fabien TARISSAN
   Fichier : impression.ml
   Date    : 09/06/04
 ***************************************************************
   Description : For the graphic printing 
 ***************************************************************)

open Agent;;
open Assembly;;
open System;;
open Graphics;;
open Param;;

(* FONCTIONS *)
(*************)

(* Dessine le role *)

let draw_role a =
  set_color ecriture;
  moveto (int_of_float a.pos_x - 1) (int_of_float a.pos_y - 12);
  draw_string (string_of_int a.role)
;;


(* Dessine le mode *)

let draw_mode a =
  match a.state with
      Active(_) ->
	set_color color_act;
	fill_circle (int_of_float a.pos_x) 
	  (int_of_float a.pos_y) size_mode;
    | Update(_) ->
	set_color color_upd;
	fill_circle (int_of_float a.pos_x) 
	  (int_of_float a.pos_y) size_mode;
    | Passive -> ()
    | Alarme(_) -> 
	set_color color_alarme;
	fill_circle (int_of_float a.pos_x) 
	  (int_of_float a.pos_y) size_circle;
    | Final ->
	set_color color_final;
	fill_circle (int_of_float a.pos_x) 
	  (int_of_float a.pos_y) size_circle;
;;


(* Dessine un agent *)
let draw_agent a =
  set_color contour;
  draw_circle (int_of_float a.pos_x) 
    (int_of_float a.pos_y) size_circle;
  draw_mode a;
  (*  draw_role a *)
;;


(* Pour afficher les connexions *)
let rec draw_link (x1,y1) l1 l2 =
  List.iter 
    (fun x ->
       moveto x1 y1;
       List.iter 
	 (fun a -> (
	    if List.mem x a.span 
	    then ( 
	      set_color link_span;
	      lineto (int_of_float a.pos_x) (int_of_float a.pos_y)
	    )
	    else if List.mem x a.cycle
	    then (
	      set_color link_cycl;
	      lineto (int_of_float a.pos_x) (int_of_float a.pos_y)
	    )
	    else ()
	  )
	 ) l2
    ) l1
;;


(* pour afficher le vecteur directeur li  l'agent *)

let draw_dir a =
  set_color black;
  moveto (int_of_float a.pos_x) (int_of_float a.pos_y);
  lineto (int_of_float (a.pos_x +. a.dx)) 
    (int_of_float (a.pos_y +. a.dy));    
;;

(* Pour afficher une liste d'agent *)

let rec draw_agent_list l = 
  match l with
      [] -> ()
    | hd::tl ->
	draw_link (int_of_float hd.pos_x , int_of_float hd.pos_y)
	  (hd.span@hd.cycle) tl;
	draw_agent hd;
	(* draw_dir hd; *)
	draw_agent_list tl
;;


(* Pour afficher la prochaine tentative *)

let draw_choice c =
  moveto rect_choice_x rect_choice_y;
  set_color fond_choix;
  fill_rect rect_choice_x rect_choice_y choice_width choice_heigth ;
  set_color contour;
  draw_rect rect_choice_x rect_choice_y choice_width choice_heigth ;
  set_color ecriture;
  let move_y = choice_heigth / 2 - 5 in
    match c with
	UnJoin -> rmoveto (choice_width /2 - 54) move_y; 
	  draw_string "Connexion cyclique"
      | BinJoin -> rmoveto (choice_width /2 - 30)  move_y; 
	  draw_string "Connexion"
      | Upd -> rmoveto (choice_width /2 - 18) move_y;
	  draw_string "Update"
      | Switch -> rmoveto (choice_width /2 - 18) move_y; 
	  draw_string "Switch"
      | Break -> rmoveto (choice_width /2 - 12) move_y;
	  draw_string "Break"
;;



(* Affiche l'tat du systme *)
let draw_system sys choice =
  clear_graph ();
  (*  draw_choice choice;*)
  draw_agent_list 
    (sys.active@sys.update@sys.passive@sys.break@sys.final);
  synchronize ()
;;



(* Initialise une fentre graphique *)

let init_graph () =
  let s = " "^(string_of_int size_window_x)^"x"
    ^(string_of_int size_window_y) 
  in
    open_graph s;
    set_color fond;
    fill_rect 0 0 size_window_x size_window_y;
    set_line_width 2;
    set_window_title "Robot"
;;


(* Pour terminer l'appilication *)

let finish_graph = close_graph ;;
